=== modified file '.bzrignore'
--- .bzrignore	2010-12-29 21:23:43 +0000
+++ .bzrignore	2010-12-30 00:52:57 +0000
@@ -337,3 +337,7 @@
 ylwrap
 docs/man
 .pc
+drizzled/message/catalog.pb.cc
+drizzled/message/catalog.pb.h
+drizzled/message/catalog_reader
+drizzled/message/catalog_writer

=== modified file 'drizzled/enum.h'
--- drizzled/enum.h	2010-12-29 21:23:43 +0000
+++ drizzled/enum.h	2010-12-30 00:52:57 +0000
@@ -1064,6 +1064,8 @@
   ER_CORRUPT_TABLE_DEFINITION_ENUM,
   ER_CORRUPT_TABLE_DEFINITION_UNKNOWN_COLLATION,
   ER_CATALOG_DOES_NOT_EXIST,
+  ER_CATALOG_CANNOT_CREATE,
+  ER_CATALOG_CANNOT_DROP,
   ER_CATALOG_NO_LOCK,
   ER_CATALOG_NO_DROP_LOCAL,
   ER_CORRUPT_CATALOG_DEFINITION,

=== modified file 'drizzled/error.cc'
--- drizzled/error.cc	2010-12-29 21:23:43 +0000
+++ drizzled/error.cc	2010-12-30 00:52:57 +0000
@@ -529,11 +529,13 @@
   ADD_ERROR_MESSAGE(EE_DISK_FULL, find(ER_DISK_FULL));
   
   // Catalog related errors
-  ADD_ERROR_MESSAGE(ER_WRONG_NAME_FOR_CATALOG, N_("Invalid catalog name"));
-  ADD_ERROR_MESSAGE(ER_CORRUPT_CATALOG_DEFINITION, N_("Corrupt or invalid catalog definition for %s : %s"));
-  ADD_ERROR_MESSAGE(ER_CATALOG_DOES_NOT_EXIST, N_("Catalog does not exist: %s"));
-  ADD_ERROR_MESSAGE(ER_CATALOG_NO_LOCK, N_("Could not gain lock on %s"));
-  ADD_ERROR_MESSAGE(ER_CATALOG_NO_DROP_LOCAL, N_("You can not drop the local catalog"));
+  ADD_ERROR_MESSAGE(ER_WRONG_NAME_FOR_CATALOG, N_("Invalid catalog name."));
+  ADD_ERROR_MESSAGE(ER_CORRUPT_CATALOG_DEFINITION, N_("Corrupt or invalid catalog definition for '%s' : '%s'."));
+  ADD_ERROR_MESSAGE(ER_CATALOG_DOES_NOT_EXIST, N_("Catalog '%s' does not exist."));
+  ADD_ERROR_MESSAGE(ER_CATALOG_CANNOT_CREATE, N_("Cannot create catalog '%s'."));
+  ADD_ERROR_MESSAGE(ER_CATALOG_CANNOT_DROP, N_("Cannot drop catalog '%s'."));
+  ADD_ERROR_MESSAGE(ER_CATALOG_NO_LOCK, N_("Could not gain lock on '%s'."));
+  ADD_ERROR_MESSAGE(ER_CATALOG_NO_DROP_LOCAL, N_("You can not drop the 'local' catalog."));
 
 
 }

=== modified file 'drizzled/plugin/catalog.cc'
--- drizzled/plugin/catalog.cc	2010-12-29 20:26:03 +0000
+++ drizzled/plugin/catalog.cc	2010-12-30 00:52:57 +0000
@@ -162,12 +162,18 @@
     return false;
   }
 
+  size_t create_count= 0;
   BOOST_FOREACH(catalog::Engine::vector::const_reference ref, Engines::singleton().catalogs())
   {
     if (ref->create(identifier, message))
-    {
-      return false;
-    }
+      create_count++;
+  }
+  assert(create_count < 2);
+
+  if (not create_count)
+  {
+    my_error(ER_CATALOG_CANNOT_CREATE, MYF(0), identifier.getName().c_str());
+    return false;
   }
 
   return true;
@@ -190,12 +196,18 @@
   }
 
   
+  size_t drop_count= 0;
   BOOST_FOREACH(catalog::Engine::vector::const_reference ref, Engines::singleton().catalogs())
   {
-    if (not ref->drop(identifier))
-    {
-      return false;
-    }
+    if (ref->drop(identifier))
+      drop_count++;
+  }
+  assert(drop_count < 2);
+
+  if (not drop_count)
+  {
+    my_error(ER_CATALOG_DOES_NOT_EXIST, MYF(0), identifier.getName().c_str());
+    return false;
   }
 
   return true;

=== added file 'drizzled/util/intrusive_ptr_base.h'
--- drizzled/util/intrusive_ptr_base.h	1970-01-01 00:00:00 +0000
+++ drizzled/util/intrusive_ptr_base.h	2010-12-30 00:52:57 +0000
@@ -0,0 +1,109 @@
+/* - mode: c; c-basic-offset: 2; indent-tabs-mode: nil; -*-
+ *  vim:expandtab:shiftwidth=2:tabstop=2:smarttab:
+ *
+ * Copyright (C) 2010 Brian Aker
+ * All rights reserved.
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions are met:
+ *     * Redistributions of source code must retain the above copyright
+ *       notice, this list of conditions and the following disclaimer.
+ *     * Redistributions in binary form must reproduce the above copyright
+ *       notice, this list of conditions and the following disclaimer in the
+ *       documentation and/or other materials provided with the distribution.
+ *     * Neither the name of the <organization> nor the
+ *       names of its contributors may be used to endorse or promote products
+ *       derived from this software without specific prior written permission.
+ *
+ * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
+ * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
+ * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
+ * DISCLAIMED. IN NO EVENT SHALL <COPYRIGHT HOLDER> BE LIABLE FOR ANY
+ * DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
+ * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
+ * LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
+ * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
+ * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
+ * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
+ *
+ */
+
+/*
+  I've looked at a number of examples on the web, this is a composite of what I have seen/liked.
+  -Brian
+*/
+
+#include <boost/checked_delete.hpp>
+#include <boost/detail/atomic_count.hpp>
+#include <boost/intrusive_ptr.hpp>
+#include <boost/thread.hpp>
+
+#ifndef DRIZZLED_UTIL_INTRUSIVE_BASE_PTR_H
+#define DRIZZLED_UTIL_INTRUSIVE_BASE_PTR_H
+
+namespace drizzled {
+namespace util {
+
+template<class T>
+class intrusive_ptr_base
+{
+public:
+  intrusive_ptr_base():
+    _reference_count(0)
+  {
+  }
+
+  intrusive_ptr_base(intrusive_ptr_base<T> const&) :
+    _reference_count(0)
+  {
+  }
+
+  intrusive_ptr_base& operator=(intrusive_ptr_base const& rhs)
+  { 
+    return *this; 
+  }
+
+  friend void intrusive_ptr_add_ref(intrusive_ptr_base<T> const* s)
+  {
+    assert(s->_reference_count >= 0);
+    assert(s != 0);
+    ++s->_reference_count;
+  }
+
+  friend void intrusive_ptr_release(intrusive_ptr_base<T> const* s)
+  {
+    assert(s->_reference_count > 0);
+    assert(s != 0);
+
+    if (--s->_reference_count == 0)
+      boost::checked_delete(static_cast<T const*>(s));
+  }
+
+  boost::intrusive_ptr<T> self()
+  { 
+    return boost::intrusive_ptr<T>((T*)this); 
+  }
+
+  boost::intrusive_ptr<const T> self() const
+  { 
+    return boost::intrusive_ptr<const T>((T const*)this); 
+  }
+
+  bool unique() const 
+  {
+    return _reference_count == 0;
+  }
+
+  int use_count() const 
+  { 
+    return _reference_count; 
+  }
+
+private:
+  mutable boost::detail::atomic_count _reference_count;
+};
+
+} // namespace util
+} // namespace drizzled
+
+#endif /* DRIZZLED_UTIL_INTRUSIVE_BASE_PTR_H */

=== modified file 'plugin/catalog/engine.cc'
--- plugin/catalog/engine.cc	2010-12-09 06:12:03 +0000
+++ plugin/catalog/engine.cc	2010-12-30 00:52:57 +0000
@@ -52,7 +52,8 @@
 
     return false;
   }
-  return false;
+
+  return true;
 }
 
 bool Engine::drop(const drizzled::identifier::Catalog &identifier)

=== added file 'plugin/catalog/tests/r/create_conflict.result'
--- plugin/catalog/tests/r/create_conflict.result	1970-01-01 00:00:00 +0000
+++ plugin/catalog/tests/r/create_conflict.result	2010-12-30 00:52:57 +0000
@@ -0,0 +1,8 @@
+SELECT CREATE_CATALOG("foo");
+CREATE_CATALOG("foo")
+1
+SELECT CREATE_CATALOG("foo");
+ERROR HY000: Cannot create catalog 'foo'.
+SELECT DROP_CATALOG("foo");
+DROP_CATALOG("foo")
+1

=== modified file 'plugin/catalog/tests/r/drop.result'
--- plugin/catalog/tests/r/drop.result	2010-12-29 20:26:03 +0000
+++ plugin/catalog/tests/r/drop.result	2010-12-30 00:52:57 +0000
@@ -9,7 +9,9 @@
 DROP_CATALOG("foo")
 1
 SELECT drop_catalog("local");
-ERROR HY000: You can not drop the local catalog
+ERROR HY000: You can not drop the 'local' catalog.
+SELECT drop_catalog("does_not_exist");
+ERROR HY000: Catalog 'does_not_exist' does not exist.
 SELECT CATALOG_NAME FROM DATA_DICTIONARY.CATALOGS WHERE CATALOG_NAME != "local";
 CATALOG_NAME
 SELECT CATALOG_NAME FROM DATA_DICTIONARY.CATALOG_CACHE WHERE CATALOG_NAME != "local";

=== added file 'plugin/catalog/tests/t/create_conflict.test'
--- plugin/catalog/tests/t/create_conflict.test	1970-01-01 00:00:00 +0000
+++ plugin/catalog/tests/t/create_conflict.test	2010-12-30 00:52:57 +0000
@@ -0,0 +1,4 @@
+SELECT CREATE_CATALOG("foo");
+--error ER_CATALOG_CANNOT_CREATE
+SELECT CREATE_CATALOG("foo");
+SELECT DROP_CATALOG("foo");

=== modified file 'plugin/catalog/tests/t/drop.test'
--- plugin/catalog/tests/t/drop.test	2010-12-29 21:23:43 +0000
+++ plugin/catalog/tests/t/drop.test	2010-12-30 00:52:57 +0000
@@ -6,5 +6,9 @@
 
 --error ER_CATALOG_NO_DROP_LOCAL
 SELECT drop_catalog("local");
+
+--error ER_CATALOG_DOES_NOT_EXIST
+SELECT drop_catalog("does_not_exist");
+
 SELECT CATALOG_NAME FROM DATA_DICTIONARY.CATALOGS WHERE CATALOG_NAME != "local";
 SELECT CATALOG_NAME FROM DATA_DICTIONARY.CATALOG_CACHE WHERE CATALOG_NAME != "local";

